#!/usr/bin/perl -w

use 5.010;
use strict;
use warnings;

use lib 'lib';

use Test::More;

use English qw( -no_match_vars );

eval 'use Test::Perl::Critic';  ## no critic
plan skip_all => 'Test::Perl::Critic required to criticise code' if $@;
Test::Perl::Critic->import( -profile => 'xt/author/.perlcritic.rc' );
all_critic_ok(
    'lib',
    't',
    'xt',
);
